Eancom::Definition.create(name: 'D96A', type: :invoic) do |document, config|
  document.header do |header|
    header.segment Eancom::Edifact::UNA.new

    header.segment Eancom::Edifact::UNB.new(
      syntax_identifier: 'UNOC',
      syntax_version_number: '3',
      interchange_sender_identification: config.header.interchange_sender_identification,
      sender_identifiction_code_qualifier: '14',
      interchange_recipient_identification: config.header.interchange_recipient_identification,
      recipient_identifiction_code_qualifier: '14',
      date: config.header.date,
      time: config.header.time,
      interchange_control_reference: config.header.interchange_control_reference,
      application_reference: config.header.application_reference,
      interchange_agreement_identifier: config.header.interchange_agreement_identifier,
      test_indicator: nil
    )
   
    header.segment Eancom::Edifact::UNH.new(
      message_reference_number: config.header.message_reference_number,
      message_type: config.header.message_type,
      message_version_number: config.header.message_version_number,
      message_release_number: config.header.message_release_number,
      controlling_agency: config.header.controlling_agency,
      association_assigned_code: config.header.association_assigned_code
    )
  end

  document.body do |body|
    messages = config.body.messages
    messages.each do |message|
      body.segment Eancom::Edifact::BGM.new(
        document_name_code: message.document_name_code,
        document_identifier: message.document_identifier,
        message_function_code: message.message_function_code
      )

      date_time_messages = message.date_time_messages
      date_time_messages.each_with_index do |dtm, index|
        next unless index == 0
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      date_time_messages.each_with_index do |dtm, index|
        next unless index == 1
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      date_time_messages.each_with_index do |dtm, index|
        next unless index == 2
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      references = message.references
      references.each_with_index do |rff, index|
        next unless index == 0
        structure = Eancom.find_structure(tag: 'RFF')
        key = rff.identifier.intern.to_sym
        body.segment Eancom::Edifact::RFF.new(
          reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
          reference_identifier: rff[key].reference_identifier
        )
      end

      references.each_with_index do |rff, index|
        next unless index == 1
        structure = Eancom.find_structure(tag: 'RFF')
        key = rff.identifier.intern.to_sym
        body.segment Eancom::Edifact::RFF.new(
          reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
          reference_identifier: rff[key].reference_identifier
        )
      end

      references.each_with_index do |rff, index|
        next unless index == 2
        structure = Eancom.find_structure(tag: 'RFF')
        key = rff.identifier.intern.to_sym
        body.segment Eancom::Edifact::RFF.new(
          reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
          reference_identifier: rff[key].reference_identifier
        )
      end

      structure = Eancom.find_structure(tag: 'NAD')
      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 0
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      references.each_with_index do |rff, index|
        next unless index == 3
        structure = Eancom.find_structure(tag: 'RFF')
        key = rff.identifier.intern.to_sym
        body.segment Eancom::Edifact::RFF.new(
          reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
          reference_identifier: rff[key].reference_identifier
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 1
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 2
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 3
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      references = message.references
      references.each_with_index do |rff, index|
        next unless index == 4

        structure = Eancom.find_structure(tag: 'RFF')
        key = rff.identifier.intern.to_sym
        body.segment Eancom::Edifact::RFF.new(
          reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
          reference_identifier: rff[key].reference_identifier
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 4
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      structure = Eancom.find_structure(tag: 'CUX')
      body.segment Eancom::Edifact::CUX.new(
        currency_usage_code_qualifier_1: structure.dictionary_lookup(:currency_usage_code_qualifier_1, message.currency.currency_usage_code_qualifier_1),
        currency_identification_code_1: message.currency.currency_identification_code_1,
        currency_type_code_qualifier_1: structure.dictionary_lookup(:currency_type_code_qualifier_1,
                                                                    message.currency.currency_type_code_qualifier_1)
      )

      structure = Eancom.find_structure(tag: 'PAT')
      body.segment Eancom::Edifact::PAT.new(
        payment_terms_type_code_qualifier: structure.dictionary_lookup(:payment_terms_type_code_qualifier,
                                                                       config.header.payment_terms_type_code_qualifier),
        payment_terms_type_description_identifier: config.header.payment_terms_type_description_identifier,
        time_reference_code: config.header.time_reference_code
      )

      # TODO: Replace hardcoded values
      structure = Eancom.find_structure(tag: 'PCD')
      body.segment Eancom::Edifact::PCD.new(
        percentage_type_code_qualifier: structure.dictionary_lookup(:percentage_type_code_qualifier,
                                                                    config.header.percentage_type_code_qualifier),
        percentage: config.header.percentage
      )

      total_quantity = 0

      items = message.items

      items.each_with_index do |item, index|

        structure = Eancom.find_structure(tag: 'LIN')
        body.segment Eancom::Edifact::LIN.new(
          line_item_identifier_1: (index + 1).to_s,
          item_identifier: item.ean,
          item_type_identification_code: structure.dictionary_lookup(:item_type_identification_code, item.item_type_identification_code)
        )

        if additional_product_id = item.additional_product_id
          additional_product_id.each do |id|
            next unless additional_information = id.additional_information

            structure = Eancom.find_structure(tag: 'PIA')
            body.segment Eancom::Edifact::PIA.new(
              product_identifier_code_qualifier: structure.dictionary_lookup(:product_identifier_code_qualifier, additional_information.product_identifier_code_qualifier),
              item_identifier_1: additional_information.item_identifier_1,
              item_type_identification_code_1: structure.dictionary_lookup(:item_type_identification_code_1,
                                                                           additional_information.item_type_identification_code_1)
            )
          end
        end

        quantities = item.quantities
        quantities.each_with_index do |qty, index|
          next unless index == 0
          structure = Eancom.find_structure(tag: 'QTY')
          key = qty.identifier.intern.to_sym
          body.segment Eancom::Edifact::QTY.new(
            quantity_type_code_qualifier: structure.dictionary_lookup(:quantity_type_code_qualifier, qty.identifier),
            quantity: qty[key].quantity
          )
        end

        quantities.each_with_index do |qty, index|
          next unless index == 1
          structure = Eancom.find_structure(tag: 'QTY')
          key = qty.identifier.intern.to_sym
          body.segment Eancom::Edifact::QTY.new(
            quantity_type_code_qualifier: structure.dictionary_lookup(:quantity_type_code_qualifier, qty.identifier),
            quantity: qty[key].quantity
          )
        end

        date_time_messages = item.date_time_messages
        date_time_messages.each do |dtm, index|
          structure = Eancom.find_structure(tag: 'DTM')
          key = dtm.identifier.intern.to_sym
          body.segment Eancom::Edifact::DTM.new(
            type: structure.dictionary_lookup(:type, dtm.identifier),
            date_time: dtm[key].date_time,
            format: '102'
          )
        end

        prices = item.prices
        prices.each do |price|
          next unless price = price.price

          structure = Eancom.find_structure(tag: 'PRI')
          body.segment Eancom::Edifact::PRI.new(
            price_code_qualifier: structure.dictionary_lookup(:price_code_qualifier, price.price_code_qualifier),
            price_amount: price.price_amount,
            price_type_code: structure.dictionary_lookup(:price_type_code, price.price_type_code),
            price_specification_code: structure.dictionary_lookup(:price_specification_code,
                                                                  price.price_specification_code)
          )
        end

        if taxes = item.taxes
          taxes.each_with_index do |tax, index|
            next unless tax = tax.tax
            next unless index == 0

            structure = Eancom.find_structure(tag: 'TAX')
            body.segment Eancom::Edifact::TAX.new(
              duty_or_tax_or_fee_function_code_qualifier: '7',
              duty_or_tax_or_fee_name_code: structure.dictionary_lookup(:duty_or_tax_or_fee_name_code,
                                                                        tax.duty_or_tax_or_fee_name_code)
            )
          end
        end

        next unless monetary_amounts = item.monetary_amounts

        monetary_amounts.each_with_index do |amount, index|
          next unless index == 0

          structure = Eancom.find_structure(tag: 'MOA')
          body.segment Eancom::Edifact::MOA.new(
            monetary_amount_type_code_qualifier: structure.dictionary_lookup(:monetary_amount_type_code_qualifier, amount.monetary_amount_type_code_qualifier),
            monetary_amount: amount.monetary_amount
          )
        end
      end
    end
  end
  document.footer do |footer|
    footer.segment Eancom::Edifact::UNS.new(
      section_identification: 'S'
    )

    messages = config.body.messages

    messages.each do |message|
      items = message.items

      items.each_with_index do |item, index|
        next unless index == items.size - 1

        counts = item.counts
        counts.each_with_index do |cnt, index|
          structure = Eancom.find_structure(tag: 'CNT')
          footer.segment Eancom::Edifact::CNT.new(
            control_total_type_code_quantifier: structure.dictionary_lookup(:control_total_type_code_quantifier, cnt.control_total_type_code_quantifier),
            control_total_value: (index + 1).to_s
          )
        end
      end
    end

    messages = config.body.messages
    messages.each do |message|
      items = message.items
      items.each do |item|
        monetary_amounts = item.monetary_amounts
        monetary_amounts.each_with_index do |amount, index|
          next unless index == items.size - 1

          monetary_amount = if amount.monetary_mount.nil?
                              ''
                            else
                              amount.monetary_amount
                            end
          structure = Eancom.find_structure(tag: 'MOA')
          footer.segment Eancom::Edifact::MOA.new(
            monetary_amount_type_code_qualifier: structure.dictionary_lookup(:monetary_amount_type_code_qualifier,
                                                                             amount.monetary_amount_type_code_qualifier),
            monetary_amount: monetary_amount
          )
        end
      end
    end

    messages = config.body.messages
    messages.each do |message|
      items = message.items
      items.each do |item|
        taxes = item.taxes
        taxes.each_with_index do |tax, index|
          next unless index == 1
          structure = Eancom.find_structure(tag: 'TAX')
          footer.segment Eancom::Edifact::TAX.new(
            duty_or_tax_or_fee_function_code_qualifier: '7',
            duty_or_tax_or_fee_name_code: structure.dictionary_lookup(:duty_or_tax_or_fee_name_code, tax.tax.duty_or_tax_or_fee_name_code)
          )
        end
      end
    end

    footer.segment Eancom::Edifact::UNT.new(
      number_of_segments_in_message: (document.total_segments + 2).to_s,
      message_reference_number: config.header.message_reference_number
    )

    footer.segment Eancom::Edifact::UNZ.new(
      interchange_control_count: messages.size.to_s,
      interchange_control_reference: config.header.interchange_control_reference.to_s
    )
  end
end
