Eancom::Definition.create(name: 'D01B', type: :invrpt) do |document, config|
  document.header do |header|
    header.segment Eancom::Edifact::UNA.new

    header.segment Eancom::Edifact::UNB.new(
      syntax_identifier: 'UNOA',
      syntax_version_number: '3',
      interchange_sender_identification: config.header.interchange_sender_identification,
      sender_identifiction_code_qualifier: '14',
      interchange_recipient_identification: config.header.interchange_recipient_identification,
      recipient_identifiction_code_qualifier: '14',
      date: config.header.date,
      time: config.header.time,
      interchange_control_reference: config.header.interchange_control_reference,
      application_reference: config.header.application_reference,
      interchange_agreement_identifier: config.header.interchange_agreement_identifier,
      test_indicator: nil
    )

    header.segment Eancom::Edifact::UNH.new(
      message_reference_number: config.header.message_reference_number,
      message_type: config.header.message_type,
      message_version_number: config.header.message_version_number,
      message_release_number: config.header.message_release_number,
      controlling_agency: config.header.controlling_agency,
      association_assigned_code: config.header.association_assigned_code
    )
  end

  document.body do |body|
    messages = config.body.messages
    messages.each do |message|
      body.segment Eancom::Edifact::BGM.new(
        document_name_code: message.document_name_code,
        document_identifier: message.document_identifier,
        message_function_code: message.message_function_code
      )
      date_time_messages = message.date_time_messages
      date_time_messages.each_with_index do |dtm, index|
        next unless index == 0
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '203'
        )
      end

      date_time_messages.each_with_index do |dtm, index|
        next unless index == 1
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 0
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9'
        )
      end
      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 1
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9'
        )
      end

      total_quantity = 0

      items = message.items

      items.each_with_index do |item, index|
        total_quantity += item.quantity.to_i

        structure = Eancom.find_structure(tag: 'LIN')
        body.segment Eancom::Edifact::LIN.new(
          line_item_identifier_1: (index + 1).to_s,
          item_identifier: item.ean,
          item_type_identification_code: structure.dictionary_lookup(:item_type_identification_code, item.item_type_identification_code)
        )

        inventories = item.inventory

        inventories.each do |inventory|
          structure = Eancom.find_structure(tag: 'INV')
          body.segment Eancom::Edifact::INV.new(
            inventory_movement_direction_code: structure.dictionary_lookup(:inventory_movement_direction_code,
                                                                           inventory.inventory_movement_direction_code),
            inventory_balance_method_code: inventory.inventory_balance_method_code
          )
        end

        quantities = item.quantities
        quantities.each_with_index do |qty, index|
          structure = Eancom.find_structure(tag: 'QTY')
          key = qty.identifier.intern.to_sym
          body.segment Eancom::Edifact::QTY.new(
            quantity_type_code_qualifier: structure.dictionary_lookup(:quantity_type_code_qualifier, qty.identifier),
            quantity: qty[key].quantity
          )
        end
      end
    end
  end

  document.footer do |footer|
    messages = config.body.messages

    footer.segment Eancom::Edifact::UNT.new(
      number_of_segments_in_message: (document.total_segments + 2).to_s,
      message_reference_number: config.header.message_reference_number
    )
    footer.segment Eancom::Edifact::UNZ.new(
      interchange_control_count: messages.size.to_s,
      interchange_control_reference: config.header.interchange_control_reference.to_s
    )
  end
end
