Eancom::Definition.create(name: 'D96A', type: :invrpt) do |document, config|
  document.header do |header|
    header.segment Eancom::Edifact::UNA.new

    header.segment Eancom::Edifact::UNB.new(
      syntax_identifier: 'UNOC',
      syntax_version_number: '3',
      interchange_sender_identification: config.header.interchange_sender_identification,
      sender_identifiction_code_qualifier: '14',
      interchange_recipient_identification: config.header.interchange_recipient_identification,
      recipient_identifiction_code_qualifier: '14',
      date: config.header.date,
      time: config.header.time,
      interchange_control_reference: config.header.interchange_control_reference,
      application_reference: config.header.application_reference,
      interchange_agreement_identifier: config.header.interchange_agreement_identifier,
      test_indicator: nil
    )

    header.segment Eancom::Edifact::UNH.new(
      message_reference_number: config.header.message_reference_number,
      message_type: config.header.message_type,
      message_version_number: config.header.message_version_number,
      message_release_number: config.header.message_release_number,
      controlling_agency: config.header.controlling_agency,
      association_assigned_code: config.header.association_assigned_code
    )
  end

  document.body do |body|
    messages = config.body.messages
    
    messages.each do |message|
      body.segment Eancom::Edifact::BGM.new(
        document_name_code: message.document_name_code,
        document_identifier: message.document_identifier,
        message_function_code: message.message_function_code
      )

      date_time_messages = message.date_time_messages
      date_time_messages.each_with_index do |dtm, index|
        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 0
        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9',
          name_and_address_description_1: nad[key].name_and_address_description_1
        )
      end

      total_quantity = 0

      items = message.items

      items.each_with_index do |item, index|
        total_quantity += item.quantity.to_i

        structure = Eancom.find_structure(tag: 'LIN')
        body.segment Eancom::Edifact::LIN.new(
          line_item_identifier_1: (index + 1).to_s,
          item_identifier: item.ean,
          item_type_identification_code: structure.dictionary_lookup(:item_type_identification_code, item.item_type_identification_code)
        )

        if additional_product_id = item.additional_product_id
          additional_product_id.each do |id|
            next unless product_identification = id.product_identification

            structure = Eancom.find_structure(tag: 'PIA')
            body.segment Eancom::Edifact::PIA.new(
              product_identifier_code_qualifier: structure.dictionary_lookup(:product_identifier_code_qualifier, product_identification.product_identifier_code_qualifier),
              item_identifier_1: product_identification.item_identifier_1,
              item_type_identification_code_1: structure.dictionary_lookup(:item_type_identification_code_1, product_identification.item_type_identification_code_1)
            )
          end
        end

        quantities = item.quantities
        quantities.each_with_index do |qty, index|
          structure = Eancom.find_structure(tag: 'QTY')
          key = qty.identifier.intern.to_sym
          body.segment Eancom::Edifact::QTY.new(
            quantity_type_code_qualifier: structure.dictionary_lookup(:quantity_type_code_qualifier, qty.identifier),
            quantity: qty[key].quantity
          )
        end

        structure = Eancom.find_structure(tag: 'LOC')
        body.segment Eancom::Edifact::LOC.new(
          location_function_qualifier: structure.dictionary_lookup(:location_function_qualifier, item.location.location_function_qualifier),
          location_name_code: item.location.location_name_code,
          code_list_identification_code: item.location.code_list_identification_code,
          code_list_responsible_agency_code: structure.dictionary_lookup(:code_list_responsible_agency_code, item.location.code_list_responsible_agency_code)
        )
      end

      body.segment Eancom::Edifact::UNT.new(
        number_of_segments_in_message: (document.total_segments + 2).to_s,
        message_reference_number: config.header.message_reference_number
      )
    end
  end

  document.footer do |footer|
    messages = config.body.messages

    footer.segment Eancom::Edifact::UNZ.new(
      interchange_control_count: messages.size.to_s,
      interchange_control_reference: config.header.interchange_control_reference.to_s
    )
  end
end
