tag = 'PCI'
Eancom.register_segment(tag: tag, klass: Eancom::Edifact::PCI)

structure = Eancom::Edifact::Structure.new(tag: tag)
#==============================================================================
# Tag                                                                         #
#==============================================================================
tag = Eancom::Edifact::Composite.new
data = Eancom::Edifact::Data.new(
  type: String,
  length: 1..3,
  dictionary: nil,
  description: 'Message Header',
  required: true
)
tag.add(:tag, data)

structure << tag
#==============================================================================
# Marking instructions code                                                   #
#==============================================================================
marking_instructions_code = Eancom::Edifact::Composite.new
data = Eancom::Edifact::Data.new(
  type: String,
  length: 1..3,
  dictionary: {
    '1' => {
      description: 'Do not mark suppliers company name',
      identifier: 'do_not_mark_suppliers_company_name'
    },
    '2' => {
      description: 'Mark customers company name',
      identifier: 'mark_customers_company_name'
    },
    '3' => {
      description: 'Mark customers references',
      identifier: 'mark_customers_references'
    },
    '6' => {
      description: 'Mark packages dimensions',
      identifier: 'mark_packages_dimensions'
    },
    '7' => {
      description: 'Mark net weight',
      identifier: 'mark_net_weight'
    },
    '8' => {
      description: 'Mark gross weight',
      identifier: 'mark_gross_weight'
    },
    '10' => {
      description: 'Mark batch number',
      identifier: 'mark_batch_number'
    },
    '14' => {
      description: 'Mark expiry date',
      identifier: 'mark_expiry_date'
    },
    '16' => {
      description: 'Buyers instructions',
      identifier: 'buyers_instructions'
    },
    '17' => {
      description: "Supplier's instructions",
      identifier: 'supliers_instructions'
    },
    '18' => {
      description: "Carrier's instructions",
      identifier: 'carriers_instructions'
    },
    '33E' => {
      description: 'Marked with serial shipping container code (GS1 Temporary Code)',
      identifier: 'marked_with_serial_shipping_container_code_gs1_temporary_code'
    },
    '34E' => {
      description: 'Marked with GS1 number (GS1 Temporary Code)',
      identifier: 'marked_with_gs1_number'
    }
  },
  description: 'Marking instructions code',
  required: false
)
marking_instructions_code.add(:marking_instructions_code, data)

structure << marking_instructions_code
#==============================================================================
# MARKS & LABELS                                                              #
#==============================================================================
marks_and_labels_code = Eancom::Edifact::Composite.new
data = Eancom::Edifact::Data.new(
  type: String,
  length: 1..35,
  dictionary: nil,
  description: 'Shipping marks description',
  required: true
)
marks_and_labels_code.add(:shipping_marks_description, data)

data = Eancom::Edifact::Data.new(
  type: String,
  length: 1..3,
  dictionary: nil,
  description: 'Container or package contents indicator code',
  required: false
)
marks_and_labels_code.add(:container_or_package_contents_indicator_mode, data)

structure << marks_and_labels_code

Eancom.register_structure(tag: 'PCI', structure: structure)
