Eancom::Definition.create(name: 'D01B', type: :slsrpt) do |document, config|
  document.header do |header|
    header.segment Eancom::Edifact::UNA.new

    header.segment Eancom::Edifact::UNB.new(
      syntax_identifier: 'UNOC',
      syntax_version_number: '3',
      interchange_sender_identification: config.header.interchange_sender_identification,
      sender_identifiction_code_qualifier: '14',
      interchange_recipient_identification: config.header.interchange_recipient_identification,
      recipient_identifiction_code_qualifier: '14',
      date: config.header.date,
      time: config.header.time,
      interchange_control_reference: config.header.interchange_control_reference
    )

    header.segment Eancom::Edifact::UNH.new(
      message_reference_number: config.header.message_reference_number,
      message_type: config.header.message_type,
      message_version_number: config.header.message_version_number,
      message_release_number: config.header.message_release_number,
      controlling_agency: config.header.controlling_agency,
      association_assigned_code: config.header.association_assigned_code
    )
  end

  document.body do |body|
    messages = config.body.messages

    messages.each do |message|
      code_list_identification_code = if message.code_list_identification_code.nil?
                                        ''
                                      else
                                        message.code_list_identification_code
                                      end
      body.segment Eancom::Edifact::BGM.new(
        document_name_code: message.document_name_code,
        code_list_identification_code: code_list_identification_code,
        code_list_responsible_agency_code: '9',
        document_identifier: message.document_identifier,
        message_function_code: message.message_function_code
      )

      date_time_messages = message.date_time_messages
      date_time_messages.each_with_index do |dtm, index|
        next unless index == 0

        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      date_time_messages.each_with_index do |dtm, index|
        next unless index == 1

        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      date_time_messages.each_with_index do |dtm, index|
        next unless index == 2

        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      nads = message.nads
      nads.each_with_index do |nad, index|
        next unless index == 0

        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9'
        )
      end

      nads.each_with_index do |nad, index|
        next unless index == 1

        structure = Eancom.find_structure(tag: 'NAD')
        key = nad.identifier.intern.to_sym
        body.segment Eancom::Edifact::NAD.new(
          party_function_code_qualifier: structure.dictionary_lookup(:party_function_code_qualifier, nad.identifier),
          party_identifier: nad[key].party_identifier,
          code_list_responsible_agency_code_1: '9'
        )
      end

      # Reference Currency
      if currency = message.currency
        structure = Eancom.find_structure(tag: 'CUX')
        body.segment Eancom::Edifact::CUX.new(
          currency_identification_code_1: currency.currency_identification_code_1,
          currency_type_code_qualifier_1: structure.dictionary_lookup(:currency_type_code_qualifier_1,
                                                                      currency.currency_type_code_qualifier_1),
          currency_usage_code_qualifier_1: structure.dictionary_lookup(:currency_usage_code_qualifier_1,
                                                                       currency.currency_usage_code_qualifier_1)
        )
      end

      # Location
      if location = message.location
        structure = Eancom.find_structure(tag: 'LOC')
        body.segment Eancom::Edifact::LOC.new(
          code_list_identification_code: location.code_list_identification_code,
          code_list_responsible_agency_code: structure.dictionary_lookup(:code_list_responsible_agency_code, location.code_list_responsible_agency_code),
          location_function_qualifier: structure.dictionary_lookup(:location_function_qualifier, location.location_function_qualifier),
          location_name_code: location.location_name_code
        )
      end

      # Sales Date
      date_time_messages.each_with_index do |dtm, index|
        next unless index == 3

        structure = Eancom.find_structure(tag: 'DTM')
        key = dtm.identifier.intern.to_sym
        body.segment Eancom::Edifact::DTM.new(
          type: structure.dictionary_lookup(:type, dtm.identifier),
          date_time: dtm[key].date_time,
          format: '102'
        )
      end

      total_quantity = 0

      items = message.items

      items.each_with_index do |item, index|
        total_quantity += item.quantity.to_i

        structure = Eancom.find_structure(tag: 'LIN')
        body.segment Eancom::Edifact::LIN.new(
          line_item_identifier_1: (index + 1).to_s,
          item_identifier: item.ean,
          item_type_identification_code: structure.dictionary_lookup(:item_type_identification_code, item.item_type_identification_code)
        )

        references = item.references
        references.each_with_index do |rff, index|
          structure = Eancom.find_structure(tag: 'RFF')
          key = rff.identifier.intern.to_sym
          body.segment Eancom::Edifact::RFF.new(
            reference_code_qualifier: structure.dictionary_lookup(:reference_code_qualifier, rff.identifier),
            reference_identifier: rff[key].reference_identifier
          )
        end

        prices = item.prices
        prices.each do |price|
          next unless price = price.price

          structure = Eancom.find_structure(tag: 'PRI')
          body.segment Eancom::Edifact::PRI.new(
            price_amount: price.price_amount,
            price_code_qualifier: structure.dictionary_lookup(:price_code_qualifier, price.price_code_qualifier),
            price_specification_code: structure.dictionary_lookup(:price_specification_code,
                                                                  price.price_specification_code),
            price_type_code: price.price_type_code
          )
        end

        qty = item.quantities
        qty.each_with_index do |qty, index|
          structure = Eancom.find_structure(tag: 'QTY')
          key = qty.identifier.intern.to_sym
          body.segment Eancom::Edifact::QTY.new(
            quantity_type_code_qualifier: structure.dictionary_lookup(:quantity_type_code_qualifier, qty.identifier),
            quantity: qty[key].quantity
          )
        end
      end

      body.segment Eancom::Edifact::UNT.new(
        number_of_segments_in_message: (document.total_segments + 2).to_s,
        message_reference_number: config.header.message_reference_number
      )
    end
  end

  document.footer do |footer|
    messages = config.body.messages

    footer.segment Eancom::Edifact::UNZ.new(
      interchange_control_count: messages.size.to_s,
      interchange_control_reference: config.header.interchange_control_reference.to_s
    )
  end
end
